@@ -9,6 +9,14 @@ mod sequence;
 #[cfg(target_os = "windows")]
 mod windows;
 
+#[cfg(target_os = "linux")]
+use crate::linux::keyboard_mouse::kimpl;
+#[cfg(target_os = "linux")]
+use crate::linux::keyboard_mouse::mimpl;
+#[cfg(target_os = "windows")]
+use crate::windows::keyboard::kimpl;
+#[cfg(target_os = "windows")]
+use crate::windows::mouse::mimpl;
 pub use keyboard::*;
 #[cfg(target_os = "linux")]
 pub use linux::*;
@@ -19,6 +27,7 @@ pub use sequence::Sequence;
 pub use windows::*;
 
 use crate::details::registry;
+use std::fmt;
 use std::sync::Arc;
 
 #[derive(Copy, Clone, Ord, PartialOrd, Hash, Eq, PartialEq, Debug)]
@@ -29,31 +38,27 @@ pub enum State {
     Released,
 }
 
-// Mouse implements.
-pub trait Button {
-    /// Send an event to Press this Button
-    fn press(&self);
-    /// Send an event to Release this Button
-    fn click(&self);
-    /// Send an event to Click (Press + Release) this key
-    fn release(&self);
-}
-
-// Keyboard implements.
-pub trait Key {
+impl Keyboard {
     /// Send an event to Press this key
-    fn press(&self);
+    pub fn press(&self) {
+        kimpl::press(*self)
+    }
+
     /// Send an event to Release this key
-    fn release(&self);
+    pub fn release(&self) {
+        kimpl::release(*self)
+    }
     /// Send an event to Click (Press + Release) this key
-    fn click(&self);
+    pub fn click(&self) {
+        kimpl::click(*self);
+    }
 
     // Some buttons are toggleable like caps lock.
     /// Whether this KeyboardKey is toggled, applies for some buttons such as Caps Lock
-    fn is_toggled(&self) -> bool;
-}
+    pub fn is_toggled(&self) -> bool {
+        kimpl::is_toggled(*self)
+    }
 
-impl Keyboard {
     /// Bind an action on this KeyboardKey, action will be invoked on a new thread.
     pub fn bind(&self, handler: impl Fn(Keyboard) + Send + Sync + 'static) {
         bind_key(*self, Action::handle_kb(handler))
@@ -76,6 +81,38 @@ impl Keyboard {
 }
 
 impl Mouse {
+    /// Send an event to Press this Button
+    pub fn press(&self) {
+        mimpl::press(*self)
+    }
+
+    /// Send an event to Release this Button
+    pub fn click(&self) {
+        mimpl::click(*self)
+    }
+
+    /// Send an event to Click (Press + Release) this key
+    pub fn release(&self) {
+        mimpl::release(*self)
+    }
+
+    pub fn move_to(x: i32, y: i32) {
+        mimpl::move_to(x, y)
+    }
+
+    pub fn move_by(x: i32, y: i32) {
+        mimpl::move_by(x, y)
+    }
+
+    pub fn click_at(&self, x: i32, y: i32) {
+        mimpl::click_at(x, y, *self);
+    }
+
+    // TODO: this does not work on linux, duh. add it.
+    pub fn track(f: impl Fn(i32, i32) + Send + Sync + 'static) {
+        registry().set_mouse_tracker(Some(Arc::new(Box::new(f))));
+    }
+
     /// Bind an action on this MouseButton, action will be invoked on a new thread.
     pub fn bind(&self, handler: impl Fn(Mouse) + Send + Sync + 'static) {
         bind_button(*self, Action::handle_mouse(handler))
@@ -102,7 +139,7 @@ impl Mouse {
 /// Whether to propagate the event for applications down the callstack.
 pub enum InhibitEvent {
     Yes,
-    Maybe(Arc<Box<dyn Fn() -> InhibitEvent + Send + Sync>>),
+    Maybe(Arc<dyn Fn() -> InhibitEvent + Send + Sync>),
     No,
 }
 
@@ -119,7 +156,7 @@ impl InhibitEvent {
     }
 
     pub fn maybe(f: impl Fn() -> InhibitEvent + Send + Sync + 'static) -> Self {
-        InhibitEvent::Maybe(Arc::new(Box::new(f)))
+        InhibitEvent::Maybe(Arc::new(f))
     }
 }
 
@@ -129,6 +166,15 @@ pub enum Event {
     Mouse(Mouse),
 }
 
+impl fmt::Display for Event {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Event::Keyboard(k) => f.write_fmt(format_args!("k ({})", k)),
+            Event::Mouse(m) => f.write_fmt(format_args!("m ({})", m)),
+        }
+    }
+}
+
 pub struct Action {
     /// What do you want to do on the key callback, see `defer` and `sequencer` to understand
     /// on which thread those are invoked.
@@ -359,3 +405,11 @@ pub fn get_state(key: &str) -> Option<String> {
 pub fn unregister_hotkey(sequence: &[Keyboard]) {
     registry().unregister_hotkey(sequence);
 }
+
+pub fn enable_debug() {
+    registry().enable_debug();
+}
+
+pub fn print_pressed_state() {
+    registry().print_pressed_state();
+}
